// logger for parallax 433MHz receiver
// only logs NUMBYTES after signal has crossed THRESH

#define RXTXPIN 7
#define RATE 1200 // for 433MHz link - 1200 to 9600

#define SYNC_INTERVAL 1000 // mills between calls to sync()
uint32_t syncTime = 0;     // time of last sync()
#define NUMBYTES 120 // number of bytes to print after signal is high
#define LEDPIN 6
#define SIGPIN 0 // analogue 0
#define THRESH 210 // was 180

int linebyte=0;
int bytecount=0;
long thistime=0;
long lasttime=0;
char name[16];
char num[16];

#include <SdFat.h>
#include <SdFatUtil.h>

Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

int incomingByte = 0;	// for incoming serial data

// store error strings in flash to save RAM
#define error(s) error_P(PSTR(s))

void error_P(const char* str) {
  // just hang, flashing a light if there's an error
  while(1) {
    digitalWrite(LEDPIN, HIGH); // light on
    delay(500);
    digitalWrite(LEDPIN, LOW); // light off
    delay(500);
  }
}

void setup() {
  pinMode(RXTXPIN, OUTPUT);
  digitalWrite(RXTXPIN, LOW); // sets the transceiver to RX
  Serial.begin(RATE); // Parallax unit is on D0 (RX)

  if (!card.init(SPI_HALF_SPEED)) error("card.init failed");

  // initialize a FAT volume
  if (!volume.init(&card)) error("volume.init failed");

  // open root directory
  if (!root.openRoot(&volume)) error("openRoot failed");
  // create a new file
  sprintf(name, "M%04dV00.TXT", RATE);
  for (uint8_t i = 0; i < 100; i++) {
    name[6] = i/10 + '0';
    name[7] = i%10 + '0';
    if (file.open(&root, name, O_CREAT | O_EXCL | O_WRITE)) break;
  }
  if (!file.isOpen()) error ("file.create");
}

void loop() {

  file.writeError = 0;

  if ((analogRead(SIGPIN) >= THRESH) && (bytecount==0)) {
    // trigger writing
    digitalWrite(LEDPIN, HIGH); // light on
    bytecount = NUMBYTES;
    thistime=millis();
    sprintf(num, "%10ld", thistime);
    file.println(); // end of record
    file.print(num);
    file.print(" ");
    sprintf(num, "%10ld", thistime-lasttime);
    file.print(num);
    file.print(" ");
    lasttime=thistime; // log delta from last record
  }

  if (Serial.available() > 0) {
    incomingByte = Serial.read();
    if (bytecount > 0) {
      bytecount--;
      sprintf(num, "%02x", incomingByte);
      file.print(num);
    }
    else {
      digitalWrite(LEDPIN, LOW); // light off
    }
  }

  if (file.writeError) error("write data failed");
  //don't sync too often - requires 2048 bytes of I/O to SD card
  if ((millis() - syncTime) <  SYNC_INTERVAL) return;
  syncTime = millis();
  if (!file.sync()) error("sync failed");

}









